function [coordinates,elements,varargout] = refineRGB(coordinates,elements,varargin)

markedElements = varargin{end};
nE = size(elements,1);

%*** Sort elements such that first edge is longest
dx = coordinates(elements(:,[2,3,1]),1)-coordinates(elements,1);
dy = coordinates(elements(:,[2,3,1]),2)-coordinates(elements,2);
[hT,idxMax] = max(reshape(dx.^2+dy.^2,nE,3),[],2);
idx = find(idxMax==2); 
elements(idx,:) = elements(idx,[2,3,1]); 
idx = find(idxMax==3); 
elements(idx,:) = elements(idx,[3,1,2]); 

[edge2nodes,element2edges,boundary2edges{1:nargin-3}] ...
    = provideEdgeData(elements,varargin{1:end-1});

%*** Mark edges for refinement
edge2newNode = zeros(max(max(element2edges)),1);
edge2newNode(element2edges(markedElements,:)) = 1;
swap = 1;
while ~isempty(swap)
    markedEdge = edge2newNode(element2edges);
    swap = find( ~markedEdge(:,1) & (markedEdge(:,2) | markedEdge(:,3)) );
    edge2newNode(element2edges(swap,1)) = 1;
end

%*** Generate new nodes
edge2newNode(find(edge2newNode)) = size(coordinates,1) + [1:nnz(edge2newNode)];
idx = find(edge2newNode);
coordinates(edge2newNode(idx),:) ...
    = (coordinates(edge2nodes(idx,1),:)+coordinates(edge2nodes(idx,2),:))/2;

%*** Refine boundary conditions
for j = 1:nargout-2;
    boundary = varargin{j};
    if ~isempty(boundary);
        newNodes = edge2newNode(boundary2edges{j});
        markedEdges = find(newNodes);
        if ~isempty(markedEdges)
            boundary = [boundary(find(~newNodes),:);
                        boundary(markedEdges,1),newNodes(markedEdges);
                        newNodes(markedEdges),boundary(markedEdges,2)];
        end
    end
    varargout(j)={boundary};
end

%*** RGB-refinement of elements
newNodes = edge2newNode(element2edges);
edge2newNode = (newNodes~=0);

%*** no edges needs refinement
idx = find(~edge2newNode(:,1));
none = elements(idx,:);

%*** green refinement (refine 1st edge)
idx = find( edge2newNode(:,1) & ~edge2newNode(:,2) & ~edge2newNode(:,3));
green = [elements(idx,3),elements(idx,1),newNodes(idx,1);
         elements(idx,2),elements(idx,3),newNodes(idx,1)];

%*** blue refinement (refine 1st and 2nd edge)
idx = find( edge2newNode(:,1) &  edge2newNode(:,2) & ~edge2newNode(:,3));
blue12 = [elements(idx,3),elements(idx,1),newNodes(idx,1);
          newNodes(idx,1),elements(idx,2),newNodes(idx,2);
          elements(idx,3),newNodes(idx,1),newNodes(idx,2)];

%*** blue refinement (refine 1st and 3rd edge)
idx = find( edge2newNode(:,1) & ~edge2newNode(:,2) &  edge2newNode(:,3));
blue13 = [elements(idx,2),elements(idx,3),newNodes(idx,1);
          newNodes(idx,1),elements(idx,3),newNodes(idx,3);
          elements(idx,1),newNodes(idx,1),newNodes(idx,3)];

%*** red refinement (refine all edges)
idx = find( edge2newNode(:,1) &  edge2newNode(:,2) &  edge2newNode(:,3));
red = [elements(idx,1),newNodes(idx,1),newNodes(idx,3);
       newNodes(idx,1),elements(idx,2),newNodes(idx,2);
       newNodes(idx,3),newNodes(idx,2),elements(idx,3);
       newNodes(idx,2),newNodes(idx,3),newNodes(idx,1)];

elements = [none;green;blue12;blue13;red];

